=begin

= https.rb -- SSL enhancement for Net::SSL.

  Copyright (C) 2001 GOTOU Yuuzou

  This program requires Net 1.2.0 or higher version.
  You can get it from RAA or Ruby's CVS repository.

== class Net::HTTP

=== Instance Methods
: use_ssl

: use_ssl=((|true_or_false|))
  sets use_ssl

: peer_cert
  certificate file to use, PEM format assumed.

: key_file
  returns private key file to use, PEM format assumed, in cert file if
  not specified but cert file is.
  
: key_file=((|path|))
  sets key_file.

: cert_file
  returns pathname of the X.509 certificate file.

: cert_file=((|path|))
  sets cert_file.

: ca_file
  returns PEM format file of CA's.

: ca_file=((|path|))
  sets ca_file.

: ca_path
  returns PEM format directory of CA's.

: ca_path=((|path|))
  sets ca_path.

=end

require 'net/http'
require 'ssl'

module Net
  module NetPrivate
    class SSLSocket < Socket
      attr_accessor :key_file, :cert_file
      attr_accessor :ca_file, :ca_path, :verify_mode

      def initialize( addr, port, otime = nil, rtime = nil, pipe = nil )
        super
        @raw_socket = @socket
        @socket = SSL::SSLSocket.new(@socket, @key_file, @cert_file)
        @soclket.ca_file = @ca_file if @ca_file
        @soclket.ca_path = @ca_path if @ca_path
      end

      def reopen(tout=nil)
        super
        @raw_socket = @socket
        @socket = SSL::SSLSocket.new(@socket, @key_file, @cert_file)
        @socket.ca_file = @ca_file if @ca_file
        @socket.ca_path = @ca_path if @ca_path
        @socket.verify_mode = @verify_mode if @verify_mode
      end

      def close
        super
        @raw_socket.close
      end

      def peer_cert; @socket.peer_cert; end
      def ssl_connect; @socket.connect; end
    end
  end

  class HTTP
    protocol_param :socket_type, ::Net::NetPrivate::SSLSocket
    attr_accessor :use_ssl
    attr_reader :peer_cert

    class Conn < ::Net::NetPrivate::HTTPRequest
      HAS_BODY=false
      METHOD="connect"

      def initialize
        super nil, nil
      end

      def exec( sock, addr, port, ver )
        @socket = sock
        request addr, port, ver
        @response = get_response
        @response
      end

      def request( addr, port, ver )
        @socket.writeline sprintf('CONNECT %s:%s HTTP/%s', addr, port, ver)
        @socket.writeline ''
      end
    end

    def on_connect
      # Initiate SSL session if use_ssl is set.
      if use_ssl
        if proxy?
          resp = Conn.new.exec(@socket, @address, @port, @http_version)
          if resp.code != '200'
            raise resp.message
          end
        end
        @socket.ssl_connect
        @peer_cert = socket.peer_cert
      end
    end

    def key_file;   @socket.key_file;  end
    def key_file=;  @socket.key_file;  end
    def cert_file;  @socket.cert_file; end
    def cert_file=; @socket.cert_file; end

    def verify_mode;  @socket.verify_mode; end
    def verify_mode=; @socket.verify_mode; end
    def ca_file;  @socket.ca_file; end
    def ca_file=; @socket.ca_file; end
    def ca_path;  @socket.ca_path; end
    def ca_path=; @socket.ca_path; end

    module ProxyMod
      def edit_path( path )
        if use_ssl
          'https://' + addr_port + path
        else
          'http://' + addr_port + path
        end
      end
    end

  end
end
