#!/usr/local/bin/ruby

require 'socket'
require 'getopts'
require 'ssl'

include SSL

STDOUT.sync = true
STDERR.sync = true
print "pid = #{$$}\n"

scheme = host = port = path = nil
p_scheme = p_host = p_port = nil

# parse request URI.
uri = ARGV[0]
if %r!(https?)://(.*?)(?::(\d+))?(/.*)! =~ uri
  scheme = $1
  host   = $2
  port   = $3 ? $3.to_i : Socket.getservbyname(scheme)
  path   = $4 || "/"
else
  STDERR.print "Invalid URI.\n"
  exit 2
end

# parse HTTP_PROXY environment variable.
if proxy = ENV['HTTP_PROXY']
  if %r!(http)://(.*?)(?::(\d+))?(/.*)! =~ proxy
    p_scheme = $1
    p_host   = $2
    p_port   = $3 ? $3.to_i : Socket.getservbyname(p_scheme)
  else
    STDERR.print "Invalid HTTP_PROXY.\n"
    exit 2
  end
end

# Connect to server.
to = proxy ? [ p_host, p_port ] : [ host, port ]
sock = TCPSocket.new(to[0], to[1])

# If scheme is ``https'' we are going to initiate SSL session.
if scheme == "https"
  # It the peer is a proxy server, send CONNECT method to
  # be switched to being a tunnel.
  if proxy
    sock.write "CONNECT #{host}:#{port} HTTP/1.0\r\n\r\n"
    while line = sock.gets
      STDERR.print line
      break if line == "\r\n"
    end
  end

  # start SSL session.
  sock = SSLSocket.new(sock)
  sock.connect
  if peer_cert = sock.peer_cert
    STDERR.print "serialNumber = #{peer_cert.serialNumber}\n"
    STDERR.print "subject      = #{peer_cert.subject}\n"
    STDERR.print "issuer       = #{peer_cert.issuer}\n"
    STDERR.print "notBefore    = #{peer_cert.notBefore}\n"
    STDERR.print "notAfter     = #{peer_cert.notAfter}\n"
  end
end

# I expect most servers accept the absoluteURI in requests.
sock.write "GET #{scheme}://#{host}:#{port}#{path} HTTP/1.0\r\n"
sock.write "Connection: close\r\n"
sock.write "\r\n"

while line = sock.gets
  STDERR.print line
  break if line == "\r\n"
end

while data = sock.read(100)
  print data
end

sock.close
